@@ -35,6 +35,7 @@
 #include "drgenius_config.h"
 #include "drgeo_gtkhelpers.h"
 #include "drgeo_scm_helper.h"
+#include "drgeo_tool.h"
 
 #define DATA_BEGIN  ((char) 2)
 #define DATA_END  ((char) 5)
@@ -84,6 +85,14 @@ drgeniusMDI::drgeniusMDI ()
   p_exportPreferencesItem = glade_xml_get_widget (xml, "exportPreferencesItem");
   p_closeItem = glade_xml_get_widget (xml, "closeItem");
   p_drgeoStatusbar = glade_xml_get_widget (xml, "drgeoStatusbar");
+  p_windowsMenu = glade_xml_get_widget (xml, "windowsMenu");
+  p_macroMenu = glade_xml_get_widget (xml, "macroMenu");
+
+  // set the submenu of macro and windows
+  gtk_menu_item_set_submenu (GTK_MENU_ITEM (p_macroMenu), gtk_menu_new ());  gtk_menu_item_set_submenu (GTK_MENU_ITEM (p_windowsMenu), gtk_menu_new ());
+
+  // tooltips for the macro menu
+  p_tooltipsMacro = gtk_tooltips_new ();
 
   reconcile_grayout_widget ();
   setMessage (_("Welcome to GNU Dr. Geo - Free Interactive geometry by OFSET"));
@@ -93,16 +102,33 @@ drgeniusMDI::drgeniusMDI ()
 void 
 drgeniusMDI::newGeometricDocument (xmlNodePtr tree)
 {
-  p_activeView = (drgeniusView *) new geoView (tree);
-  p_viewList = addView (p_activeView);
+  drgeniusView *tmpView;
+  GList *tmpList;
+
+  tmpView = (drgeniusView *) new geoView (tree);
+  tmpList = addView (tmpView);
+
+  if (tmpList != NULL)
+    {
+      p_activeView = tmpView;
+      p_viewList = tmpList;
+    }
 }
  
 void 
 drgeniusMDI::newTextDocument (xmlNodePtr tree)
 {
-  p_activeView = (drgeniusView *) new editorView (tree);
-  p_viewList = addView (p_activeView);
+  drgeniusView *tmpView;
+  GList *tmpList;
 
+  tmpView = (drgeniusView *) new editorView (tree);
+  tmpList = addView (tmpView);
+
+  if (tmpList != NULL)
+    {
+      p_activeView = tmpView;
+      p_viewList = tmpList;
+    }
 }
 
 void
@@ -136,7 +162,7 @@ drgeniusMDI::openSession (gchar *filename)
 				       filename, g_strerror (errno));
       gtk_dialog_run (GTK_DIALOG (dialog));
       gtk_widget_destroy (dialog);
-
+      return;
     }
   tree = tree->children;
   /* vistit the top child */
@@ -193,6 +219,9 @@ drgeniusMDI::closeView (drgeniusView *v)
 void
 drgeniusMDI::renameView (drgeniusView *v, const gchar *newName)
 {
+  GtkWidget *w;
+  gint n;
+
   gtk_notebook_set_tab_label_text (GTK_NOTEBOOK (p_noteBook),
                                    v->widgetView (),
                                    newName);
@@ -200,6 +229,11 @@ drgeniusMDI::renameView (drgeniusView *v, const gchar *newName)
                                     v->widgetView (),
                                     newName);
   v->setName (newName);
+
+  // Rename the menu as well
+  n = gtk_notebook_page_num (GTK_NOTEBOOK (p_noteBook), v->widgetView ());
+  w = gtk_bin_get_child (GTK_BIN (getMenuItem (n)));
+  gtk_label_set_text (GTK_LABEL (w), newName);
 }
 
 drgeniusView *
@@ -274,6 +308,7 @@ drgeniusMDI::reconcile_grayout_widget ()
 	  gtk_widget_set_sensitive (p_gridItem, TRUE);
 	  gtk_widget_set_sensitive (p_gridItemSc, TRUE);
 	  gtk_widget_set_sensitive (p_printFigure, TRUE);
+	  gtk_widget_set_sensitive (p_macroMenu, TRUE);
 	}
       else if (p_activeView->childType() == EDITOR_CHILD)
 	{
@@ -285,6 +320,7 @@ drgeniusMDI::reconcile_grayout_widget ()
 	  gtk_widget_set_sensitive (p_gridItem, FALSE);
 	  gtk_widget_set_sensitive (p_gridItemSc, FALSE);
 	  gtk_widget_set_sensitive (p_printFigure, FALSE);
+	  gtk_widget_set_sensitive (p_macroMenu, FALSE);
 	}
     }
   else
@@ -303,43 +339,62 @@ drgeniusMDI::reconcile_grayout_widget ()
       gtk_widget_set_sensitive (p_saveAsItem, FALSE);
       gtk_widget_set_sensitive (p_saveItem, FALSE);
       gtk_widget_set_sensitive (p_printFigure, FALSE);
+      gtk_widget_set_sensitive (p_macroMenu, FALSE);
+
     }
 }
 
 GList * 
 drgeniusMDI::addView (drgeniusView *v)
 {
-  GtkWidget *label;
+  GtkWidget *label, *w;
   int curr_page;
 
-  if (p_viewList == NULL)
+  if (v->name ())
     {
-      // First remove the NullView
-      gtk_container_remove (GTK_CONTAINER (p_drgeoMDIVbox),
-			    p_nullViewWidget);
-      p_noteBook = gtk_notebook_new ();
-      gtk_notebook_set_scrollable (GTK_NOTEBOOK (p_noteBook), true);
-      gtk_notebook_popup_enable (GTK_NOTEBOOK (p_noteBook));
-      gtk_notebook_set_tab_pos (GTK_NOTEBOOK (p_noteBook), GTK_POS_BOTTOM);
-      gtk_box_pack_start_defaults (GTK_BOX (p_drgeoMDIVbox), p_noteBook);
-      gtk_box_reorder_child (GTK_BOX (p_drgeoMDIVbox), p_noteBook, 2);
-
-      // set a signal when page is changed
-      g_signal_connect (G_OBJECT (p_noteBook), "switch-page",
-			 G_CALLBACK(on_notebook_switchPage),
-			 (gpointer) this);
+      if (p_viewList == NULL)
+	{
+	  // First remove the NullView
+	  gtk_container_remove (GTK_CONTAINER (p_drgeoMDIVbox),
+				p_nullViewWidget);
+	  p_noteBook = gtk_notebook_new ();
+	  gtk_notebook_set_scrollable (GTK_NOTEBOOK (p_noteBook), true);
+	  gtk_notebook_popup_enable (GTK_NOTEBOOK (p_noteBook));
+	  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (p_noteBook), GTK_POS_BOTTOM);
+	  gtk_box_pack_start_defaults (GTK_BOX (p_drgeoMDIVbox), p_noteBook);
+	  gtk_box_reorder_child (GTK_BOX (p_drgeoMDIVbox), p_noteBook, 2);
+	  
+	  // set a signal when page is changed
+	  g_signal_connect (G_OBJECT (p_noteBook), "switch-page",
+			    G_CALLBACK(on_notebook_switchPage),
+			    (gpointer) this);
+	  
+	  gtk_widget_show (p_noteBook);
+	}
       
-      gtk_widget_show (p_noteBook);
+      label = gtk_label_new ((const gchar *) v->name ());
+      curr_page = gtk_notebook_get_current_page (GTK_NOTEBOOK (p_noteBook));
+      gtk_notebook_insert_page (GTK_NOTEBOOK (p_noteBook), v->widgetView (),
+				label, curr_page + 1);
+      gtk_notebook_next_page (GTK_NOTEBOOK (p_noteBook));
+      
+      // Insert a menu item in the Windows menu
+      w = gtk_menu_item_get_submenu (GTK_MENU_ITEM (p_windowsMenu));
+      label = gtk_menu_item_new_with_label ((const gchar *) v->name ());
+      gtk_menu_shell_insert (GTK_MENU_SHELL (w), label, curr_page + 1);
+      gtk_widget_show (label);
+      g_signal_connect (G_OBJECT (label), "activate",
+			G_CALLBACK (on_windowsMenu_cb),
+			(gpointer) v);
+    
+      p_activeView = v;
+      return g_list_insert (p_viewList, (gpointer) v, curr_page + 1);
+    }
+  else
+    {
+      delete v;
+      return NULL;
     }
-
-  label = gtk_label_new ((const gchar *) v->name ());
-  curr_page = gtk_notebook_get_current_page (GTK_NOTEBOOK (p_noteBook));
-  gtk_notebook_insert_page (GTK_NOTEBOOK (p_noteBook), v->widgetView (),
-                            label, curr_page + 1);
-  gtk_notebook_next_page (GTK_NOTEBOOK (p_noteBook));
-  
-  p_activeView = v;
-  return g_list_append (p_viewList, (gpointer) v);
 }
 
 GList *
@@ -355,6 +410,10 @@ drgeniusMDI::removeView (drgeniusView *v)
 
   delete v;
 
+  // Remove the menu item in the toolbar
+  gtk_container_remove 
+    (GTK_CONTAINER (gtk_menu_item_get_submenu (GTK_MENU_ITEM (p_windowsMenu))), getMenuItem (n));
+  
   if (tmp == NULL)
     {
       gtk_container_remove (GTK_CONTAINER (p_drgeoMDIVbox),
@@ -404,6 +463,15 @@ drgeniusMDI::setActiveView (gint page)
   reconcile_grayout_widget ();
 }
 
+void 
+drgeniusMDI::setActiveView (drgeniusView *v)
+{
+  gint page;
+
+  page = g_list_index (p_viewList, (gconstpointer) v);
+  if (page != -1)
+    gtk_notebook_set_current_page (GTK_NOTEBOOK (p_noteBook), page); 
+}
 
 GtkWindow *
 drgeniusMDI::mainWindow ()
@@ -417,6 +485,77 @@ drgeniusMDI::setTransientDialog (GtkWindow *dialog)
   gtk_window_set_transient_for (dialog, p_mainWindow);
 }
 
+GtkWidget *
+drgeniusMDI::getMenuItem (gint n)
+{
+  GList *children;
+  gpointer w;
+
+  children = gtk_container_get_children 
+    (GTK_CONTAINER (gtk_menu_item_get_submenu (GTK_MENU_ITEM (p_windowsMenu))));
+  w = g_list_nth_data (children, n);
+  g_list_free (children);
+
+  return GTK_WIDGET (w);
+}
+
+void 
+drgeniusMDI::addMacroMenuItem (gchar *name, gchar *description)
+{
+  GtkWidget *w, *item, *tips;
+  w = gtk_menu_item_get_submenu (GTK_MENU_ITEM (p_macroMenu));
+  item = gtk_menu_item_new_with_label ((const gchar *) name);
+  gtk_menu_append (GTK_MENU_SHELL (w), item);
+  gtk_widget_show (item);
+
+  gtk_tooltips_set_tip (GTK_TOOLTIPS (p_tooltipsMacro), item,
+			description, NULL);
+
+  g_signal_connect (G_OBJECT (item), "activate",
+		    G_CALLBACK (on_macroMenu_cb),
+		    (gpointer) NULL);
+
+}
+
+void drgeniusMDI::
+setMacroMenuItemDescription (gchar *name, gchar *description)
+{
+  GList *children, *p;
+  GtkWidget *menuItem, *label;
+  GtkTooltipsData* tips;
+  const gchar *aName;
+
+  children = gtk_container_get_children 
+    (GTK_CONTAINER 
+     (gtk_menu_item_get_submenu (GTK_MENU_ITEM (p_macroMenu))));
+  p = children;
+  while (p)
+    {
+      menuItem = GTK_WIDGET (p->data);
+      label = gtk_bin_get_child (GTK_BIN (menuItem));
+      aName = gtk_label_get_text (GTK_LABEL (label));
+      if (!strcmp (name, aName))
+	  break;
+      p = g_list_next (p);
+    }
+  if (p == NULL)
+    g_printerr ("Cannot not find macro %s in the macro menu\n",name);
+  else
+    {
+      tips = gtk_tooltips_data_get (menuItem);
+      if (tips)
+      {
+	g_free(tips->tip_text);
+	tips->tip_text = g_strdup (description);
+      }
+      else
+	gtk_tooltips_set_tip (GTK_TOOLTIPS (p_tooltipsMacro), menuItem,
+			      description, NULL);
+    }
+  g_list_free (children);
+}
+
+
 gboolean 
 on_drgeoMain_delete_event (GtkWidget *widget, GdkEvent *event,
 			   gpointer user_data)
@@ -1753,6 +1892,40 @@ on_notebook_switchPage (GtkNotebook *notebook, GtkNotebookPage *page,
   mdi->setActiveView (p);
 }
 
+void on_windowsMenu_cb (GtkWidget * widget, gpointer v)
+{
+  mdi->setActiveView ((drgeniusView *) v);
+}
+
+void on_macroMenu_cb (GtkWidget * widget, gpointer v)
+{
+  drgeoFigure *figure;
+  drgeoGtkDrawable *area;
+  GtkWidget *label;
+  const gchar *text;
+
+  // go in macro mode
+  if (mdi->activeView ()->childType () != FIGURE_CHILD)
+    {
+      g_printerr ("drgenius_mdi::on_macroMenu_cb not a FIGURE_CHILD while accessing macro-construction menu\n");
+      return;
+    }
+  figure = ((geoView *) mdi->activeView ())->figure ();
+  area = (drgeoGtkDrawable *) figure->getDrawable ();
+  area->setTool (area->macroPlayToolDialogLess);
+  figure->setMode (MACRO_PLAY_MODE, NULL);
+  mdi->setMessage (_("Play a recorded macro-construction"));
+
+  // Switch the macroPlayToolDialogLess to the selected macro from the
+  // menu
+  label = gtk_bin_get_child (GTK_BIN (widget));
+  text = gtk_label_get_text (GTK_LABEL (label));
+  if (!((drgeoMacroPlayToolDialogLess *)area->macroPlayToolDialogLess)
+      ->selectMacro (text))
+    g_printerr ("drgenius_mdi::on_macroMenu_cb can't select macro %s\n",
+		text);
+}
+
 static gboolean
 is_clist_row_selected (GtkCList * clist, gint row)
 {
