@@ -496,6 +496,8 @@ drgeoFigure::undo (gint level)
       selection.vide ();
       underMouse.vide ();
       highlightItem.vide ();
+      if (itemFilter)
+	itemFilter->reset ();
       doNotDraw.vide ();
       ((drgeoCommand *) undoLevel->data)->unexecute ();
       undoLevel = prev;
@@ -517,6 +519,8 @@ drgeoFigure::redo (gint level)
       underMouse.vide ();
       highlightItem.vide ();
       doNotDraw.vide ();
+      if (itemFilter)
+	itemFilter->reset ();
       ((drgeoCommand *) next->data)->execute ();
       undoLevel = next;
     }
@@ -588,13 +592,13 @@ drgeoFigure::addToUndoQueue (drgeoCommand * command)
 gboolean
 drgeoFigure::undoActive ()
 {
-  return (gboolean) (undoLevel->data);
+  return (gboolean) (undoLevel->data != 0);
 }
 
 gboolean
 drgeoFigure::redoActive ()
 {
-  return (gboolean) (undoLevel->next);
+  return (gboolean) (undoLevel->next != 0);
 }
 
 void
