@@ -740,6 +740,59 @@ drgeoMacroPlayTool::finish (drgeoDrawableUI * drawable)
   delete player;
 }
 
+
+// -----------------------------------------------------
+// Dialog Less Macro Play Tool, used for macro execution.
+// From the menu barre
+// -----------------------------------------------------
+
+void
+drgeoMacroPlayToolDialogLess::initialize (drgeoDrawableUI * drawable)
+{
+  // First invoke the initialize() method of drgeoBuildTool
+  // super-class.
+  drgeoBuildTool::initialize (drawable);
+
+  // Now let's do our stuff.
+  player = new drgeoMacroPlayer (drawable->getFigure ());
+
+  // Instruct the drawable about the Player.
+  drawable->setMacroPlayer (player);
+}
+
+// All the other methods are inherited from drgeoBuildTool.  We only
+// redefine the handleChoice() method, and of course the initialize()
+// and finish().
+
+void 
+drgeoMacroPlayToolDialogLess::
+handleChoice (drgeoDrawableUI * drawable, geometricObject * item)
+{
+  // Forward this item to the macro Player.
+  if (player && item)
+    {
+      player->add (item);
+    }
+}
+
+void
+drgeoMacroPlayToolDialogLess::
+finish (drgeoDrawableUI * drawable)
+{
+  // First invoke the initialize() method of drgeoPlayTool
+  // super-class.
+  drgeoBuildTool::finish (drawable);
+
+  // Free the drgeoMacroPlayer.
+  delete player;
+}
+
+gboolean drgeoMacroPlayToolDialogLess::
+selectMacro (const gchar *name)
+{
+  return player->setMacro (name);
+}
+
 // -----------------------------------------------------
 // Delete Tool, used for object deletion.
 // -----------------------------------------------------
